@@ -1,4 +1,4 @@
-# Copyright © 2019 Felix Lechner
+# Copyright (C) 2019 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, you can find it on the World Wide
-# Web at http://www.gnu.org/copyleft/gpl.html, or write to the Free
+# Web at https://www.gnu.org/copyleft/gpl.html, or write to the Free
 # Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 # MA 02110-1301, USA
 
@@ -20,33 +20,32 @@ package Test::Lintian::Output::Universal;
 
 =head1 NAME
 
-Test::Lintian::Output::Universal -- routines to process universal tags
+Test::Lintian::Output::Universal -- routines to process universal hints
 
 =head1 SYNOPSIS
 
-  use Test::Lintian::Output::Universal qw(get_tagnames);
+  use Test::Lintian::Output::Universal qw(get_tag_names);
 
-  my $filepath = "path to a universal tag file";
-  my @tags = get_tagnames($filepath);
+  my $filepath = "path to a universal hint file";
+  my @tags = get_tag_names($filepath);
 
 =head1 DESCRIPTION
 
-Helper routines to deal with universal tags and tag files. This is an
+Helper routines to deal with universal hints and hint files. This is an
 abstract format that has the minimum information found in all Lintian
 output formats.
 
 =cut
 
-use strict;
+use v5.20;
 use warnings;
-use autodie;
-use v5.10;
+use utf8;
 
 use Exporter qw(import);
 
 BEGIN {
     our @EXPORT_OK = qw(
-      get_tagnames
+      get_tag_names
       order
       package_name
       package_type
@@ -57,42 +56,51 @@ BEGIN {
 }
 
 use Carp;
+use Const::Fast;
+use List::SomeUtils qw(uniq);
 use List::Util qw(all);
 use Path::Tiny;
+use Unicode::UTF8 qw(encode_utf8);
 
-use constant SPACE => q{ };
-use constant EMPTY => q{};
-use constant COLON => q{:};
-use constant LPARENS => q{(};
-use constant RPARENS => q{)};
-use constant NEWLINE => qq{\n};
+const my $SPACE => q{ };
+const my $COLON => q{:};
+const my $LPARENS => q{(};
+const my $RPARENS => q{)};
 
 =head1 FUNCTIONS
 
 =over 4
 
-=item get_tagnames(PATH)
+=item get_tag_names(PATH)
 
-Gets all the tag names mentioned in universal tag file located
+Gets all the tag names mentioned in universal hint file located
 at PATH.
 
 =cut
 
-sub get_tagnames {
+sub get_tag_names {
     my ($path) = @_;
 
     my @lines = path($path)->lines_utf8({ chomp => 1 });
     my @names = map { tag_name($_) } @lines;
 
-    return @names;
+    return uniq @names;
 }
 
+=item order
+
+=cut
+
 sub order {
     my ($line) = @_;
 
     return package_type($line) . $line;
 }
 
+=item package_name
+
+=cut
+
 sub package_name {
     my ($line) = @_;
 
@@ -100,6 +108,10 @@ sub package_name {
     return $package;
 }
 
+=item package_type
+
+=cut
+
 sub package_type {
     my ($line) = @_;
 
@@ -107,6 +119,10 @@ sub package_type {
     return $type;
 }
 
+=item tag_name
+
+=cut
+
 sub tag_name {
     my ($line) = @_;
 
@@ -114,31 +130,39 @@ sub tag_name {
     return $name;
 }
 
+=item parse_line
+
+=cut
+
 sub parse_line {
     my ($line) = @_;
 
     my ($package, $type, $name, $details)
       = $line =~ qr/^(\S+)\s+\(([^)]+)\):\s+(\S+)(?:\s+(.*))?$/;
 
-    croak "Cannot parse line $line"
+    croak encode_utf8("Cannot parse line $line")
       unless all { length } ($package, $type, $name);
 
     return ($package, $type, $name, $details);
 }
 
+=item universal_string
+
+=cut
+
 sub universal_string {
     my ($package, $type, $name, $details) = @_;
 
-    croak 'Need a package name'
+    croak encode_utf8('Need a package name')
       unless length $package;
-    croak 'Need a package type'
+    croak encode_utf8('Need a package type')
       unless length $type;
-    croak 'Need a tag name'
+    croak encode_utf8('Need a tag name')
       unless length $name;
 
     my $line
-      = $package . SPACE . LPARENS . $type . RPARENS . COLON . SPACE . $name;
-    $line .= SPACE . $details
+      = $package. $SPACE. $LPARENS. $type. $RPARENS. $COLON. $SPACE. $name;
+    $line .= $SPACE . $details
       if length $details;
 
     return $line;
@@ -146,6 +170,14 @@ sub universal_string {
 
 =back
 
+=head1 AUTHOR
+
+Originally written by Felix Lechner <felix.lechner@lease-up.com> for Lintian.
+
+=head1 SEE ALSO
+
+lintian(1)
+
 =cut
 
 1;
